/* Code by Kevin Turner - kevin@kevinturner.net
 * Created for Herman Aguinis (CU Denver)
 * Summer 2005
 * 
 * Program takes up to ten sets of numerical input. These sets are then used to 
 * calculated an f squared value.
 */

import java.awt.Component; // Component in addComponent
import java.awt.Graphics; // Graphics in paint
import javax.swing.*; // JApplet, JLabel, etc
import java.awt.*; // Container and Layout
import java.awt.event.*; // ActionListener
import java.awt.print.*; // Printing
import java.text.DecimalFormat; // format output

public class EffectSize extends JApplet implements Printable
{
  Container theContainer;
  GridBagLayout mainLayout;
  GridBagConstraints mainConstraints;
  Font fontNormal, fontBold, fontLarge, fontSmall;
  JLabel titleL, numOfSetsL, chooseL, sigmaL, nL, sL, rhoOrBetaL, spacer, fSquaredL;
  JTextField[] sigma, n, s, rhoOrBeta;
  JTextField fSquared;
  int numCols, numOfActiveSets;
  JButton calculate, clear, print;
  ButtonGroup rhoOrBetaBG, numOfSetsBG;
  JRadioButton rhoB, betaB;
  JRadioButton[] numOfSetsB;
  ButtonHandler handler;
  EnterListener eListener;
  double[] sigmaV, nV, sV, rhoV;
  DecimalFormat df;
  
  public void init()
  {
    // set layout
    theContainer = getContentPane();
    mainLayout = new GridBagLayout();
    mainConstraints = new GridBagConstraints();
    theContainer.setLayout(mainLayout);
    
    // initialize fonts
    fontNormal = new Font("Arial", Font.PLAIN, 12);
    fontBold = new Font("Arial", Font.BOLD, 12);
    fontLarge = new Font("Arial", Font.BOLD, 16);
    fontSmall = new Font("Arial", Font.BOLD, 10);
    
    // set up DecimalFormat
    df = new DecimalFormat("0.################"); // for f squared outpu
    
    // set JLabels
    titleL = new JLabel("<html>Computation of Effect Size: Modified &#402;<sup>2</sup></html>");
    titleL.setFont(fontLarge);
    titleL.setHorizontalAlignment(JLabel.CENTER);
    titleL.setOpaque(true);
    titleL.setBackground(Color.LIGHT_GRAY);
    chooseL = new JLabel("<html>&#961;&nbsp;or &#946;:&nbsp;</html>");
    chooseL.setFont(fontBold);
    chooseL.setHorizontalAlignment(JLabel.RIGHT);
    chooseL.setOpaque(true);
    chooseL.setBackground(Color.LIGHT_GRAY);
    numOfSetsL = new JLabel(" # of Categories: ");
    numOfSetsL.setFont(fontBold);
    numOfSetsL.setHorizontalAlignment(JLabel.RIGHT);
    numOfSetsL.setOpaque(true);
    numOfSetsL.setBackground(Color.LIGHT_GRAY);
    sigmaL = new JLabel("<html>&#963;<sub>i</sub>:&nbsp;</html>");
    sigmaL.setFont(fontBold);
    sigmaL.setHorizontalAlignment(JLabel.RIGHT);
    sigmaL.setOpaque(true);
    sigmaL.setBackground(Color.LIGHT_GRAY);
    nL = new JLabel("<html>n<sub>i</sub>:&nbsp;</html>");
    nL.setFont(fontBold);
    nL.setHorizontalAlignment(JLabel.RIGHT);
    nL.setOpaque(true);
    nL.setBackground(Color.LIGHT_GRAY);
    sL = new JLabel("<html>s<sub>x<sub>i</sub></sub>:&nbsp;</html>");
    sL.setFont(fontBold);
    sL.setHorizontalAlignment(JLabel.RIGHT);
    sL.setOpaque(true);
    sL.setBackground(Color.LIGHT_GRAY);
    rhoOrBetaL = new JLabel("<html>&#961;<sub>i</sub>:&nbsp;</html>");
    rhoOrBetaL.setFont(fontBold);
    rhoOrBetaL.setHorizontalAlignment(JLabel.RIGHT);
    rhoOrBetaL.setOpaque(true);
    rhoOrBetaL.setBackground(Color.LIGHT_GRAY);
    spacer = new JLabel(" ");
    spacer.setOpaque(true);
    spacer.setBackground(Color.LIGHT_GRAY);
    fSquaredL = new JLabel("<html>&#402;<sup>2</sup>:&nbsp;</html>");
    fSquaredL.setFont(fontBold);
    fSquaredL.setHorizontalAlignment(JLabel.RIGHT);
    fSquaredL.setOpaque(true);
    fSquaredL.setBackground(Color.LIGHT_GRAY);
    
    // add JLabels
    addComponent(titleL, theContainer, mainLayout, mainConstraints, 0,0,11,1, 
                  GridBagConstraints.BOTH, GridBagConstraints.CENTER);
    addComponent(chooseL, theContainer, mainLayout, mainConstraints, 0,1,1,1, 
                  GridBagConstraints.BOTH, GridBagConstraints.EAST);
    addComponent(numOfSetsL, theContainer, mainLayout, mainConstraints, 0,2,1,1, 
                  GridBagConstraints.BOTH, GridBagConstraints.EAST);
    addComponent(sigmaL, theContainer, mainLayout, mainConstraints, 0,3,1,1, 
                  GridBagConstraints.BOTH, GridBagConstraints.EAST);
    addComponent(nL, theContainer, mainLayout, mainConstraints, 0,4,1,1, 
                  GridBagConstraints.BOTH, GridBagConstraints.EAST);
    addComponent(sL, theContainer, mainLayout, mainConstraints, 0,5,1,1, 
                  GridBagConstraints.BOTH, GridBagConstraints.EAST);
    addComponent(rhoOrBetaL, theContainer, mainLayout, mainConstraints, 0,6,1,1, 
                  GridBagConstraints.BOTH, GridBagConstraints.EAST);
    addComponent(spacer, theContainer, mainLayout, mainConstraints, 0,7,1,1, 
                  GridBagConstraints.BOTH, GridBagConstraints.EAST);
    addComponent(fSquaredL, theContainer, mainLayout, mainConstraints, 0,8,1,1, 
                  GridBagConstraints.BOTH, GridBagConstraints.EAST);
    
    // set JTextFields
    numCols = 5;
    
    sigma = new JTextField[10];
    for(int i = 0; i < sigma.length; i++)
      sigma[i] = new JTextField(numCols);
      
    n = new JTextField[10];
    for(int i = 0; i < n.length; i++)
      n[i] = new JTextField(numCols);
      
    s = new JTextField[10];
    for(int i = 0; i < s.length; i++)
      s[i] = new JTextField(numCols);
      
    rhoOrBeta = new JTextField[10];
    for(int i = 0; i < rhoOrBeta.length; i++)
      rhoOrBeta[i] = new JTextField(numCols);
      
    fSquared = new JTextField(numCols * 2);
    fSquared.setEditable(false);
    fSquared.setBackground(Color.LIGHT_GRAY);
    
    // add JTextFields
    for(int i = 0; i < sigma.length; i++)
    {
      addComponent(sigma[i], theContainer, mainLayout, mainConstraints, i+1,3,1,1, 
                    GridBagConstraints.NONE, GridBagConstraints.EAST);
      sigma[i].setNextFocusableComponent(n[i]);
      addComponent(n[i], theContainer, mainLayout, mainConstraints, i+1,4,1,1, 
                    GridBagConstraints.NONE, GridBagConstraints.EAST);
      n[i].setNextFocusableComponent(s[i]);
      addComponent(s[i], theContainer, mainLayout, mainConstraints, i+1,5,1,1, 
                    GridBagConstraints.NONE, GridBagConstraints.EAST);
      s[i].setNextFocusableComponent(rhoOrBeta[i]);
      addComponent(rhoOrBeta[i], theContainer, mainLayout, mainConstraints, i+1,6,1,1, 
                    GridBagConstraints.NONE, GridBagConstraints.EAST);
      if(i < sigma.length-1)
        rhoOrBeta[i].setNextFocusableComponent(sigma[i+1]);
    }
    addComponent(fSquared, theContainer, mainLayout, mainConstraints, 1,8,2,1, 
                    GridBagConstraints.NONE, GridBagConstraints.SOUTH);
    
    // set and add JButtons
    calculate = new JButton("Calculate");
    calculate.setPreferredSize(new Dimension(175,25));
    addComponent(calculate, theContainer, mainLayout, mainConstraints, 1,7,3,1, 
                  GridBagConstraints.NONE, GridBagConstraints.SOUTH);
    clear = new JButton("Clear All");
    clear.setPreferredSize(new Dimension(115, 25));
    addComponent(clear, theContainer, mainLayout, mainConstraints, 4,7,2,1, 
                  GridBagConstraints.NONE, GridBagConstraints.SOUTH);
    print = new JButton("Print");
    print.setPreferredSize(new Dimension(115, 25));
    addComponent(print, theContainer, mainLayout, mainConstraints, 6,7,2,1, 
                  GridBagConstraints.NONE, GridBagConstraints.SOUTH);
    
    // set and add ButtonGroups and JRadioButtons
    rhoOrBetaBG = new ButtonGroup();
    rhoB = new JRadioButton("<html>&#961;</html>", true);
    rhoB.setBackground(Color.WHITE);
    betaB = new JRadioButton("<html>&#946;</html>", false);
    betaB.setBackground(Color.WHITE);
    rhoOrBetaBG.add(rhoB);
    rhoOrBetaBG.add(betaB);
    addComponent(rhoB, theContainer, mainLayout, mainConstraints, 1,1,1,1, 
                  GridBagConstraints.NONE, GridBagConstraints.CENTER);
    addComponent(betaB, theContainer, mainLayout, mainConstraints, 2,1,1,1, 
                  GridBagConstraints.NONE, GridBagConstraints.CENTER);
    
    numOfSetsBG = new ButtonGroup();              
    numOfSetsB = new JRadioButton[10];
    for(int i = 0; i < numOfSetsB.length; i++)
    {
      numOfSetsB[i] = new JRadioButton(String.valueOf(i+1), false);
      numOfSetsB[i].setBackground(Color.WHITE);
      numOfSetsB[i].setActionCommand(String.valueOf(i+1));
      numOfSetsBG.add(numOfSetsB[i]);
      addComponent(numOfSetsB[i], theContainer, mainLayout, mainConstraints, i+1,2,1,1, 
                    GridBagConstraints.NONE, GridBagConstraints.CENTER);
    }
    numOfSetsB[0].setEnabled(false); // 1 set is not valid
    
    // initialize value arrays
    sigmaV = new double[10];
    nV = new double[10];
    sV = new double[10];
    rhoV = new double[10];
    
    // add listeners
    handler = new ButtonHandler();
    calculate.addActionListener(handler);
    clear.addActionListener(handler);
    print.addActionListener(handler);
    rhoB.addActionListener(handler);
    betaB.addActionListener(handler);
    for(int i = 0; i < numOfSetsB.length; i++)
      numOfSetsB[i].addActionListener(handler);
    
    eListener = new EnterListener();
    for(int i = 0; i < rhoOrBeta.length; i++)
      rhoOrBeta[i].addActionListener(eListener);
    
    // select largest number of sets (and set numOfActiveSets)
    numOfSetsB[numOfSetsB.length-1].doClick();
  }
  
  public void start()
  {
  }

  public void paint(Graphics g)
  {
    theContainer.paintComponents(g);
  }
  
  public void stop()
  {
  }
  
  // adds specified components to specified container
  void addComponent(Component componentToAdd, Container targetContainer,
                      GridBagLayout targetLayout, GridBagConstraints targetConstraints,
                      int col, int row, int colSpan, int rowSpan, int fill, int anchor)
  {
  targetConstraints.weightx = 1;
  targetConstraints.weighty = 1;

  targetConstraints.gridx = col;
  targetConstraints.gridy = row;
  targetConstraints.gridwidth = colSpan;
  targetConstraints.gridheight = rowSpan;

  targetConstraints.fill = fill;
  targetConstraints.anchor = anchor;

  targetLayout.setConstraints(componentToAdd, targetConstraints);
  targetContainer.add(componentToAdd);
  }
  
  // validates input
  public boolean validateInput()
  {
    int i = 0;
    double temp = 0.0;
    int tempI = 0;
    JLabel currentLabel = new JLabel();
    String errorMessage = "";
    
    // get values and convert to doubles
    try
    {
      for(i = 0; i < numOfActiveSets; i++)
      {
        currentLabel = sigmaL;
        sigmaV[i] = Double.parseDouble(sigma[i].getText());
        currentLabel = nL;
        nV[i] = Double.parseDouble(n[i].getText());
        
        // test constraints on n: must be an integer greater than 0
        if(nV[i] <= 0 || nV[i]%1 != 0)
        {
          errorMessage = "<html>Invalid input: " + 
                        currentLabel.getText().replaceAll("<html>","").replaceAll(":&nbsp;</html>","").replaceAll("i",String.valueOf(i+1)) + 
                        " must be an integer greater than 0</html>";
          JOptionPane.showMessageDialog(this, errorMessage, "Input Error", JOptionPane.ERROR_MESSAGE);
          return false;
        }
        
        currentLabel = sL;
        sV[i] = Double.parseDouble(s[i].getText());
        currentLabel = rhoOrBetaL;
        temp = Double.parseDouble(rhoOrBeta[i].getText());
        
        if(rhoB.isSelected()) // if input is rho
          rhoV[i] = temp;
        else // if input is beta
          rhoV[i] = calculateRho(temp, i);
        
        // test constraints on rho: must be >= -1 and <= 1
        if(rhoV[i] < -1 || rhoV[i] > 1)
        {
          if(rhoB.isSelected()) // if input is rho
          {
            errorMessage = "<html>Invalid input: " + 
                          currentLabel.getText().replaceAll("<html>","").replaceAll(":&nbsp;</html>","").replaceAll("i",String.valueOf(i+1)) + 
                          " must be greater than or equal to -1 and less than or equal to 1</html>";
          }
          else // if input is beta
          {
            errorMessage = "<html>Invalid input: " + 
                          currentLabel.getText().replaceAll("<html>","").replaceAll(":&nbsp;</html>","").replaceAll("i",String.valueOf(i+1)) + 
                          " is invalid</html>";
          }
          
          JOptionPane.showMessageDialog(this, errorMessage, "Input Error", JOptionPane.ERROR_MESSAGE);
          return false;
        }
      }
    }
    catch(NumberFormatException e) // if invalid values
    {
      errorMessage = "<html>Invalid input: " + 
                        currentLabel.getText().replaceAll("<html>","").replaceAll(":&nbsp;</html>","").replaceAll("i",String.valueOf(i+1)) + 
                        " is not a valid number</html>";
      JOptionPane.showMessageDialog(this, errorMessage, "Input Error", JOptionPane.ERROR_MESSAGE);
      return false;
    }
    
    return(true);
  }
  
  // calculates rho from beta
  public double calculateRho(double beta, int index)
  {
    return((beta * sV[index]) / sigmaV[index]);
  }
  
  // calculates and returns f squared
  public double calculateFSquared()
  {
    double answer = 0.0;
    double sum1 = 0.0;
    double sum2 = 0.0;
    double sum3 = 0.0;
    double sum4 = 0.0;
    
    // calculate sums
    for(int i = 0; i < numOfActiveSets; i++)
    {
      sum1 += ((nV[i] - 1) * Math.pow(rhoV[i],2) * Math.pow(sigmaV[i],2));
      sum2 += ((nV[i] - 1) * rhoV[i] * sigmaV[i] * sV[i]);
      sum3 += ((nV[i] - 1) * Math.pow(sV[i],2));
      sum4 += ((nV[i] - 2) * Math.pow(sigmaV[i],2) * (1 - Math.pow(rhoV[i],2)));
    }
    
    // calculate f squared
    answer = (sum1 - (Math.pow(sum2,2) / sum3)) / sum4;
    
    return(answer);
  }
  
  // for printing
  public int print(Graphics g, PageFormat pf, int pageIndex)
  {
    if (pageIndex > 0)
      return Printable.NO_SUCH_PAGE;
    
    Graphics2D g2d = (Graphics2D)g;
    g2d.translate(pf.getImageableX(), pf.getImageableY());
    g2d.scale(.6, .6);
    paint(g2d);
    return Printable.PAGE_EXISTS;
  }
  
  // for printing
  public void doPrint()
  {
    PrinterJob pjob = PrinterJob.getPrinterJob();
    PageFormat pf = pjob.defaultPage();
    pjob.setPrintable(this, pf);
    try
    {
      if (pjob.printDialog())
        pjob.print();
    }
    catch (PrinterException err) {}
  }
  
  // handles button clicks
  private class ButtonHandler implements ActionListener
  {
    AbstractButton source;
    
    public void actionPerformed(ActionEvent e)
    {
      try
      {
        source = (AbstractButton)e.getSource();
      }
      catch(ClassCastException err) { return; }; // not a button
      
      if(source == calculate)
      {
        // validate input, return if not valid
        if(!validateInput())
          return;
        
        // calculate and display f squared
        fSquared.setText(String.valueOf(df.format(calculateFSquared())));
        
        // updateUI
        fSquared.updateUI();
      }
      else if(source == clear)
      {
        for(int i = 0; i < sigma.length; i++)
          sigma[i].setText("");
      
        for(int i = 0; i < n.length; i++)
          n[i].setText("");
      
        for(int i = 0; i < s.length; i++)
          s[i].setText("");
      
        for(int i = 0; i < rhoOrBeta.length; i++)
          rhoOrBeta[i].setText("");
        
        fSquared.setText("");
      }
      else if(source == print)
      {
        doPrint();
      }
      else if(source == rhoB)
      {
        rhoOrBetaL.setText("<html>&#961;<sub>i</sub>:&nbsp;</html>");
      }
      else if(source == betaB)
      {
        rhoOrBetaL.setText("<html>&#946;<sub>i</sub>:&nbsp;</html>");
      }
      else
      {
        // get which radio button is selected
        numOfActiveSets = (int)Integer.parseInt((source).getActionCommand());
        
        // set all inputs to editable(true)
        for(int i = 0; i < sigma.length; i++)
        {
          sigma[i].setEditable(true);
          sigma[i].setBackground(Color.WHITE);
          sigma[i].setFocusable(true);
          n[i].setEditable(true);
          n[i].setBackground(Color.WHITE);
          n[i].setFocusable(true);
          s[i].setEditable(true);
          s[i].setBackground(Color.WHITE);
          s[i].setFocusable(true);
          rhoOrBeta[i].setEditable(true);
          rhoOrBeta[i].setBackground(Color.WHITE);
          rhoOrBeta[i].setFocusable(true);
        }
        
        // set appropriate inputs to editable(false)
        for(int i = numOfActiveSets; i < numOfSetsB.length; i++)
        {
          for(int j = i; j < sigma.length; j++)
          {
            sigma[j].setEditable(false);
            sigma[j].setText("");
            sigma[j].setBackground(Color.GRAY);
            sigma[j].setFocusable(false);
          }
          for(int k = i; k < n.length; k++)
          {
            n[k].setEditable(false);
            n[k].setText("");
            n[k].setBackground(Color.GRAY);
            n[k].setFocusable(false);
          }
          for(int x = i; x < s.length; x++)
          {
            s[x].setEditable(false);
            s[x].setText("");
            s[x].setBackground(Color.GRAY);
            s[x].setFocusable(false);
          }
          for(int y = i; y < rhoOrBeta.length; y++)
          {
            rhoOrBeta[y].setEditable(false);
            rhoOrBeta[y].setText("");
            rhoOrBeta[y].setBackground(Color.GRAY);
            rhoOrBeta[y].setFocusable(false);
          }
        }
      }
    }
  }
  
  // listens for enter press
  private class EnterListener implements ActionListener 
  {
    JTextField source;
    JComponent nextFC;
    
    public void actionPerformed(ActionEvent e)
    {
      try
      {
        source = (JTextField)e.getSource();
        nextFC = (JComponent)source.getNextFocusableComponent();
        
        // if last editable rhoOrBeta
        if(nextFC == null || !nextFC.isFocusable())
          calculate.doClick();
      }
      catch(ClassCastException err) { return; }; // cannot cast
    }
  }
}